@@ -7,17 +7,11 @@
 import os
 import re
 
-
-try:
-    import tidy
-    have_utidy = True
-except ImportError:
-    import mx.Tidy
-    have_utidy = False
-
 from config import java
 from util import newTempfile, runcmd
 from elementtree.ElementTree import parse, tostring        
+from htmlentitydefs import name2codepoint
+from BeautifulSoup import BeautifulSoup
 
 from zope.interface import implements
 from interfaces import IHTML2FOConverter
@@ -27,20 +21,39 @@ dirname = os.path.dirname(__file__)
 
 def tidyhtml(filename, encoding):
 
-    # Tidy does not like uppercase names or names containg '-'
-    encoding = encoding.lower().replace('-', '')
-    options = dict(drop_empty_paras=1,
-                   char_encoding=encoding,
-                   clean=1,
-                   output_xhtml=1,
-                   add_xml_decl=1)
-                                  
     html = open(filename, 'rb').read()
 
-    if have_utidy:
-        html = tidy.parseString(html, **options)
-    else:
-        nerrors, nwarnings, html, errordata = mx.Tidy.tidy(html, **options)
+    # use BeautifulSoup for performing HTML checks
+    # and conversion to XHTML
+    soup = BeautifulSoup(html)
+
+    # check if all image files exist
+    for img in soup.findAll('img'):
+        src = img['src']
+        if not os.path.exists(src):
+            raise IOError('No image file found: %s' % src)
+
+    html = str(soup.prettify())
+
+    # replace the HTML tag
+    html = '<html xmlns="http://www.w3.org/1999/xhtml">' + \
+            html[html.find('<html') + 6:]
+    # add the XML preamble
+    html = '<?xml version="1.0" ?>\n' + html
+
+    # replace all HTML entities with numeric entities
+    def handler(mo):
+        """ Callback to convert entities """
+        e = mo.group(1)
+        v = e[1:-1]
+        if not v.startswith('#'):
+            codepoint =  name2codepoint.get(v)
+            return codepoint and '&#%d;' % codepoint or ''
+        else:
+            return e
+    
+    entity_reg = re.compile('(&.*?;)')
+    html = entity_reg.sub(handler, html)
 
     filename = newTempfile()
     open(filename, 'wb').write(str(html))
@@ -66,7 +79,6 @@ class HTML2FOConverter(object):
 
         cmd = '"%s"' % java + \
               ' -Xms128m -Xmx128m -jar "%(csstoxslfo)s" "%(filename)s" -fo "%(fo_filename)s" country=%(country)s language=%(language)s' % vars()
-
         status, output = runcmd(cmd)
         if status != 0:
             raise RuntimeError('Error executing: %s' % cmd)
@@ -101,7 +113,7 @@ class HTML2FOConverter(object):
 
                 value = get(attr)
                 if value == '0':
-                    node.attrib[attr]= '0em'
+                    node.attrib[attr] = '0em'
 
             if get('page-break-after') == 'always':
                 del node.attrib['page-break-after']
